@@ -1,4 +1,4 @@
-/* $Id: autowhite.c,v 1.36 2004/08/02 12:11:48 manu Exp $ */
+/* $Id: autowhite.c,v 1.41 2005/10/04 19:15:55 manu Exp $ */
 
 /*
  * Copyright (c) 2004 Emmanuel Dreyfus
@@ -32,7 +32,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID
-__RCSID("$Id: autowhite.c,v 1.36 2004/08/02 12:11:48 manu Exp $");
+__RCSID("$Id: autowhite.c,v 1.41 2005/10/04 19:15:55 manu Exp $");
 #endif
 #endif
 
@@ -58,10 +58,11 @@ __RCSID("$Id: autowhite.c,v 1.36 2004/08/02 12:11:48 manu Exp $");
 #include <arpa/inet.h>
 
 #include "conf.h"
-#include "except.h"
 #include "pending.h"
 #include "dump.h"
 #include "autowhite.h"
+#include "acl.h"
+#include "sync.h"
 
 struct autowhitelist autowhite_head;
 pthread_rwlock_t autowhite_lock;
@@ -96,6 +97,7 @@ autowhite_add(sa, salen, from, rcpt, date, queueid)
 	time_t autowhite_validity;
 	int h, mn, s;
 	int dirty = 0;
+	ipaddr *mask = NULL;
 
 	if ((autowhite_validity = conf.c_autowhite_validity) == 0)
 		return;
@@ -111,6 +113,17 @@ autowhite_add(sa, salen, from, rcpt, date, queueid)
 	if (!iptostring(sa, salen, addr, sizeof(addr)))
 		return;
 
+	switch (sa->sa_family) {
+	case AF_INET:
+		mask = (ipaddr *)&conf.c_match_mask;
+		break;
+#ifdef AF_INET6
+	case AF_INET6:
+		mask = (ipaddr *)&conf.c_match_mask6;
+		break;
+#endif
+	}
+
 	AUTOWHITE_WRLOCK;
 	for (aw = TAILQ_FIRST(&autowhite_head); aw; aw = next_aw) {
 		next_aw = TAILQ_NEXT(aw, a_list);
@@ -136,10 +149,10 @@ autowhite_add(sa, salen, from, rcpt, date, queueid)
 		/*
 		 * Look for an already existing entry
 		 */
-		if (ip_equal(sa, aw->a_sa) &&
+		if (ip_match(sa, aw->a_sa, mask) &&
 		    ((conf.c_lazyaw == 1) ||
-		    ((strncasecmp(from, aw->a_from, ADDRLEN) == 0) &&
-		    (strncasecmp(rcpt, aw->a_rcpt, ADDRLEN) == 0)))) {
+		    ((strcasecmp(from, aw->a_from) == 0) &&
+		    (strcasecmp(rcpt, aw->a_rcpt) == 0)))) {
 			timeradd(&now, &delay, &aw->a_tv);
 
 			dirty++;
@@ -181,6 +194,7 @@ autowhite_check(sa, salen, from, rcpt, queueid)
 {
 	struct autowhite *aw;
 	struct autowhite *next_aw;
+	struct pending *pending;
 	struct timeval now, delay;
 	char addr[IPADDRSTRLEN];
 	time_t autowhite_validity;
@@ -202,6 +216,17 @@ autowhite_check(sa, salen, from, rcpt, queueid)
 	if (!iptostring(sa, salen, addr, sizeof(addr)))
 		return EXF_NONE;
 
+	switch (sa->sa_family) {
+	case AF_INET:
+		mask = (ipaddr *)&conf.c_match_mask;
+		break;
+#ifdef AF_INET6
+	case AF_INET6:
+		mask = (ipaddr *)&conf.c_match_mask6;
+		break;
+#endif
+	}
+
 	AUTOWHITE_WRLOCK;
 	for (aw = TAILQ_FIRST(&autowhite_head); aw; aw = next_aw) {
 		next_aw = TAILQ_NEXT(aw, a_list);
@@ -229,27 +254,14 @@ autowhite_check(sa, salen, from, rcpt, queueid)
 		/*
 		 * Look for our record
 		 */
-		switch (sa->sa_family) {
-		case AF_INET:
-			mask = (ipaddr *)&conf.c_match_mask;
-			break;
-#ifdef AF_INET6
-		case AF_INET6:
-			mask = (ipaddr *)&conf.c_match_mask6;
-			break;
-#endif
-		}
 		if (ip_match(sa, aw->a_sa, mask) &&
 		    ((conf.c_lazyaw == 1) ||
-		    ((strncasecmp(from, aw->a_from, ADDRLEN) == 0) &&
-		    (strncasecmp(rcpt, aw->a_rcpt, ADDRLEN) == 0)))) {
+		    ((strcasecmp(from, aw->a_from) == 0) &&
+		    (strcasecmp(rcpt, aw->a_rcpt) == 0)))) {
 			timeradd(&now, &delay, &aw->a_tv);
 
 			dirty++;
 
-			syslog(LOG_INFO, "%s: addr %s from %s rcpt %s: "
-				"autowhitelisted for more %02d:%02d:%02d",
-				queueid, addr, from, rcpt, h, mn, s);
 			break;
 		}
 	}
@@ -258,10 +270,25 @@ autowhite_check(sa, salen, from, rcpt, queueid)
 	if (dirty != 0)
 		dump_dirty += dirty;
 
-	if (aw != NULL) 
-		return EXF_AUTO;	
+	if (aw == NULL) 
+		return EXF_NONE;
 
-	return EXF_NONE;
+	syslog(LOG_INFO, "%s: addr %s from %s rcpt %s: "
+		"autowhitelisted for more %02d:%02d:%02d",
+		queueid, addr, from, rcpt, h, mn, s);
+	/*
+	 * We need to tell our peers about this, we use a
+	 * fictive pending record
+	 */
+	PENDING_WRLOCK;
+	pending = pending_get(sa, salen, from, rcpt, 
+	    (time_t)0);
+	if (pending != NULL) {
+		peer_delete(pending);
+		pending_put(pending);
+	}
+	PENDING_UNLOCK;
+	return EXF_AUTO;	
 }
 
 int
@@ -275,7 +302,7 @@ autowhite_textdump(stream)
 	struct tm tm;
 
 	fprintf(stream, "\n\n#\n# Auto-whitelisted tuples\n#\n");
-	fprintf(stream, "# Sender IP    %32s    %32s    Expire\n",
+	fprintf(stream, "# Sender IP\t%s\t%s\tExpire\n",
 	    "Sender e-mail", "Recipient e-mail");
 
 	AUTOWHITE_RDLOCK;
@@ -286,7 +313,7 @@ autowhite_textdump(stream)
 		iptostring(aw->a_sa, aw->a_salen, textaddr, sizeof(textaddr));
 
 		fprintf(stream, 
-		    "%s     %32s    %32s    %ld AUTO # %s\n",
+		    "%s\t%s\t%s\t%ld AUTO # %s\n",
 		    textaddr, aw->a_from, aw->a_rcpt, 
 		    (long)aw->a_tv.tv_sec, textdate);
 
@@ -320,17 +347,15 @@ autowhite_get(sa, salen, from, rcpt, date) /* autowhite list must be locked */
 
 	bzero((void *)aw, sizeof(*aw));
 
-	if ((aw->a_sa = malloc(salen)) == NULL) {
+	if ((aw->a_sa = malloc(salen)) == NULL ||
+	    (aw->a_from = strdup(from)) == NULL ||
+	    (aw->a_rcpt = strdup(rcpt)) == NULL) {
 		syslog(LOG_ERR, "malloc failed: %s", strerror(errno));
 		exit(EX_OSERR);
 	}
 
 	memcpy(aw->a_sa, sa, salen);
 	aw->a_salen = salen;
-	strncpy(aw->a_from, from, ADDRLEN);
-	aw->a_from[ADDRLEN] = '\0';
-	strncpy(aw->a_rcpt, rcpt, ADDRLEN);
-	aw->a_rcpt[ADDRLEN] = '\0';
 
 	if (date == NULL)
 		timeradd(&now, &delay, &aw->a_tv);
@@ -348,6 +373,8 @@ autowhite_put(aw)	/* autowhite list must be write-locked */
 {
 	TAILQ_REMOVE(&autowhite_head, aw, a_list);	
 	free(aw->a_sa);
+	free(aw->a_from);
+	free(aw->a_rcpt);
 	free(aw);
 
 	return;
