@@ -1,4 +1,4 @@
-/* $Id: conf.h,v 1.26 2004/08/08 21:24:20 manu Exp $ */
+/* $Id: conf.h,v 1.31 2005/03/19 07:38:53 manu Exp $ */
 
 /*
  * Copyright (c) 2004 Emmanuel Dreyfus
@@ -60,9 +60,12 @@ struct conf {
 	int c_cold;
 	int c_forced;
 	int c_debug;
+	int c_acldebug;
 	int c_quiet;
 	int c_noauth;
+	int c_noaccessdb;
 	int c_nospf;
+	int c_delayedreject;
 	int c_testmode;
 	int c_delay;
 	int c_autowhite_validity;
@@ -81,25 +84,27 @@ struct conf {
 	int c_lazyaw;
 	int c_dumpfreq;
 	int c_timeout;
+	int c_extendedregex;
 };
 
 /* c_forced flags */
-#define C_NONE		0x0000
-#define C_DEBUG		0x0001
-#define C_QUIET		0x0002
-#define C_NOAUTH	0x0004
-#define C_NOSPF		0x0008 
-#define C_TESTMODE	0x0010
-#define C_DELAY		0x0020
-#define C_AUTOWHITE	0x0040
-#define C_PIDFILE	0x0080
-#define C_DUMPFILE	0x0100
-#define C_MATCHMASK	0x0200
-#define C_SOCKET	0x0400
-#define C_USER		0x0800
-#define C_NODETACH	0x1000
-#define C_LAZYAW	0x2000
-#define C_MATCHMASK6	0x4000
+#define C_GLNONE	0x00000
+#define C_DEBUG		0x00001
+#define C_QUIET		0x00002
+#define C_NOAUTH	0x00004
+#define C_NOSPF		0x00008 
+#define C_TESTMODE	0x00010
+#define C_DELAY		0x00020
+#define C_AUTOWHITE	0x00040
+#define C_PIDFILE	0x00080
+#define C_DUMPFILE	0x00100
+#define C_MATCHMASK	0x00200
+#define C_SOCKET	0x00400
+#define C_USER		0x00800
+#define C_NODETACH	0x01000
+#define C_LAZYAW	0x02000
+#define C_MATCHMASK6	0x04000
+#define C_ACLDEBUG	0x08000
 #define C_NOTFORCED(x) 	((conf.c_forced & (x)) == 0) 
 
 /* c_report */
