@@ -1,4 +1,4 @@
-/* $Id: milter-greylist.c,v 1.94 2004/08/02 20:17:38 manu Exp $ */
+/* $Id: milter-greylist.c,v 1.111 2005/09/20 14:00:35 manu Exp $ */
 
 /*
  * Copyright (c) 2004 Emmanuel Dreyfus
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID  
-__RCSID("$Id: milter-greylist.c,v 1.94 2004/08/02 20:17:38 manu Exp $");
+__RCSID("$Id: milter-greylist.c,v 1.111 2005/09/20 14:00:35 manu Exp $");
 #endif
 #endif
 
@@ -48,6 +48,7 @@ __RCSID("$Id: milter-greylist.c,v 1.94 2004/08/02 20:17:38 manu Exp $");
 #include <errno.h>
 #include <fcntl.h>
 #include <pwd.h>
+#include <grp.h>
 #include <unistd.h>
 
 /* On IRIX, <unistd.h> defines a EX_OK that clashes with <sysexits.h> */
@@ -67,7 +68,7 @@ __RCSID("$Id: milter-greylist.c,v 1.94 2004/08/02 20:17:38 manu Exp $");
 #include <libmilter/mfapi.h>
 
 #include "dump.h"
-#include "except.h"
+#include "acl.h"
 #include "conf.h"
 #include "pending.h"
 #include "sync.h"
@@ -78,6 +79,8 @@ __RCSID("$Id: milter-greylist.c,v 1.94 2004/08/02 20:17:38 manu Exp $");
 static char *strncpy_rmsp(char *, char *, size_t);
 static char *gmtoffset(time_t *, char *, size_t);
 static void writepid(char *);
+static void log_and_report_greylisting(SMFICTX *ctx, 
+    struct mlfi_priv *priv, char *rcpt);
 
 struct smfiDesc smfilter =
 {
@@ -138,12 +141,12 @@ mlfi_connect(ctx, hostname, addr)
 #endif
 		default:
 			priv->priv_elapsed = 0;
-			priv->priv_whitelist = EXF_NONIPV4;
+			priv->priv_whitelist = EXF_WHITELIST | EXF_NONIPV4;
 			break;
 		}
 	} else {
 		priv->priv_elapsed = 0;
-		priv->priv_whitelist = EXF_NONIPV4;
+		priv->priv_whitelist = EXF_WHITELIST | EXF_NONIPV4;
 	}
 
 	return SMFIS_CONTINUE;
@@ -206,16 +209,13 @@ mlfi_envfrom(ctx, envfrom)
 
 	/*
 	 * Reload the config file if it has been touched
-	 * Only one thread at a time, please.
 	 */
-	CONF_WRLOCK;
 	conf_update();
-	CONF_UNLOCK;
 
 	/*
 	 * Is the sender non-IPv4?
 	 */
-	if (priv->priv_whitelist == EXF_NONIPV4)
+	if (priv->priv_whitelist & EXF_NONIPV4)
 		return SMFIS_CONTINUE;
 
 	/*
@@ -227,7 +227,7 @@ mlfi_envfrom(ctx, envfrom)
 		    "User %s authenticated, bypassing greylisting", 
 		    auth_authen);
 		priv->priv_elapsed = 0;
-		priv->priv_whitelist = EXF_AUTH;
+		priv->priv_whitelist = EXF_WHITELIST | EXF_AUTH;
 
 		return SMFIS_CONTINUE;
 	} 
@@ -243,21 +243,7 @@ mlfi_envfrom(ctx, envfrom)
 		    "STARTTLS succeeded for DN=\"%s\", bypassing greylisting", 
 		    cert_subject);
 		priv->priv_elapsed = 0;
-		priv->priv_whitelist = EXF_STARTTLS;
-
-		return SMFIS_CONTINUE;
-	}
-
-	/*
-	 * Is the sender IP or its e-mail address in the
-	 * permanent whitelist? We do this before SPF as
-	 * a lookup in the whitelist is less expensive
-	 * than a DNS lookup.
-	 */
-	if ((priv->priv_whitelist = except_sender_filter(SA(&priv->priv_addr),
-	    priv->priv_addrlen, priv->priv_hostname, priv->priv_from, 
-	    priv->priv_queueid)) != EXF_NONE) {
-		priv->priv_elapsed = 0;
+		priv->priv_whitelist = EXF_WHITELIST | EXF_STARTTLS;
 
 		return SMFIS_CONTINUE;
 	}
@@ -270,12 +256,16 @@ mlfi_envfrom(ctx, envfrom)
 	    priv->priv_helo, *envfrom) != EXF_NONE)) {
 		char ipstr[IPADDRLEN + 1];
 
-		inet_ntop(AF_INET, &priv->priv_addr, ipstr, IPADDRLEN);
-		syslog(LOG_DEBUG, 
-		    "Sender IP %s and address %s are SPF-compliant, "
-		    "bypassing greylist", ipstr, *envfrom);
+		if (iptostring(SA(&priv->priv_addr),
+		    priv->priv_addrlen, ipstr, sizeof(ipstr))) {
+
+			syslog(LOG_DEBUG, 
+			    "Sender IP %s and address %s are SPF-compliant, "
+			    "bypassing greylist", ipstr, *envfrom);
+		}
+
 		priv->priv_elapsed = 0;
-		priv->priv_whitelist = EXF_SPF;
+		priv->priv_whitelist = EXF_WHITELIST | EXF_SPF;
 
 		return SMFIS_CONTINUE;
 	}
@@ -290,10 +280,9 @@ mlfi_envrcpt(ctx, envrcpt)
 {
 	struct mlfi_priv *priv;
 	time_t remaining;
-	char hdr[HDRLEN + 1];
+	char *greylist;
 	char addrstr[IPADDRSTRLEN];
 	char rcpt[ADDRLEN + 1];
-	int h, mn, s;
 
 	priv = (struct mlfi_priv *) smfi_getpriv(ctx);
 
@@ -305,6 +294,21 @@ mlfi_envrcpt(ctx, envrcpt)
 		syslog(LOG_DEBUG, "%s: addr = %s, from = %s, rcpt = %s", 
 		    priv->priv_queueid, addrstr, priv->priv_from, *envrcpt);
 
+	 /*
+	  * If sendmail rules have defined a ${greylist} macro
+	  * with value WHITE, then it is whitelisted
+	  */
+	if ((conf.c_noaccessdb == 0) &&
+	    ((greylist = smfi_getsymval(ctx, "{greylist}")) != NULL) &&
+	    (strcmp(greylist, "WHITE") == 0)) {
+		syslog(LOG_DEBUG, 
+		    "whitelisted by {greylist}");
+		priv->priv_elapsed = 0;
+		priv->priv_whitelist = EXF_ACCESSDB;
+ 
+		return SMFIS_CONTINUE;
+	}
+
 	/*
 	 * For multiple-recipients messages, if the sender IP or the
 	 * sender e-mail address is whitelisted, authenticated, or
@@ -318,13 +322,13 @@ mlfi_envrcpt(ctx, envrcpt)
 	 * header explaining that they were whitelisted, whereas some
 	 * of them would not.
 	 */
-	if ((priv->priv_whitelist == EXF_ADDR) ||
-	    (priv->priv_whitelist == EXF_DOMAIN) ||
-	    (priv->priv_whitelist == EXF_FROM) ||
-	    (priv->priv_whitelist == EXF_AUTH) ||
-	    (priv->priv_whitelist == EXF_SPF) ||
-	    (priv->priv_whitelist == EXF_NONIPV4) ||
-	    (priv->priv_whitelist == EXF_STARTTLS))
+	if ((priv->priv_whitelist & EXF_ADDR) ||
+	    (priv->priv_whitelist & EXF_DOMAIN) ||
+	    (priv->priv_whitelist & EXF_FROM) ||
+	    (priv->priv_whitelist & EXF_AUTH) ||
+	    (priv->priv_whitelist & EXF_SPF) ||
+	    (priv->priv_whitelist & EXF_NONIPV4) ||
+	    (priv->priv_whitelist & EXF_STARTTLS))
 		return SMFIS_CONTINUE;
 
 	/* 
@@ -339,10 +343,11 @@ mlfi_envrcpt(ctx, envrcpt)
 	rcpt[ADDRLEN] = '\0';
 
 	/*
-	 * Check if the recipient e-mail is in the permanent whitelist.
+	 * Check the ACL
 	 */
-	if ((priv->priv_whitelist = except_rcpt_filter(rcpt,
-	    priv->priv_queueid)) != EXF_NONE) {
+	if ((priv->priv_whitelist = acl_filter(SA(&priv->priv_addr),
+	    priv->priv_addrlen, priv->priv_hostname, priv->priv_from,
+	    rcpt, priv->priv_queueid)) & EXF_WHITELIST) {
 		priv->priv_elapsed = 0;
 		return SMFIS_CONTINUE;
 	}
@@ -371,7 +376,7 @@ mlfi_envrcpt(ctx, envrcpt)
 	 * for everyone, it will not go to mlfi_eom(), and priv_whitelist 
 	 * will not be used.
 	 */
-	priv->priv_whitelist = EXF_RCPT;
+	priv->priv_whitelist = EXF_WHITELIST | EXF_RCPT;
 
 	/*
 	 * Check if the tuple {sender IP, sender e-mail, recipient e-mail}
@@ -383,29 +388,27 @@ mlfi_envrcpt(ctx, envrcpt)
 	    priv->priv_queueid) != 0)
 		return SMFIS_CONTINUE;
 
+	priv->priv_remaining = remaining;
+
 	/*
 	 * The message has been added to the greylist and will be delayed.
-	 * Log this and report to the client.
+	 * If the sender address is null, this will be done after the DATA
+	 * phase, otherwise immediately.
 	 */
-	h = remaining / 3600;
-	remaining = remaining % 3600;
-	mn = (remaining / 60);
-	remaining = remaining % 60;
-	s = remaining;
-
-	syslog(LOG_INFO, "%s: addr %s from %s to %s delayed for %02d:%02d:%02d",
-	    priv->priv_queueid, addrstr, priv->priv_from, *envrcpt, h, mn, s);
-
-	if (conf.c_quiet) {
-		(void)smfi_setreply(ctx, "451", "4.7.1", 
-		    "Greylisting in action, please come back later");
-	} else {
-		snprintf(hdr, HDRLEN, 
-		    "Greylisting in action, please come "
-		    "back in %02d:%02d:%02d", h, mn, s);
-		(void)smfi_setreply(ctx, "451", "4.7.1", hdr);
+	if ((conf.c_delayedreject == 1) && 
+	    (strcmp(priv->priv_from, "<>") == 0)) {
+		priv->priv_delayed_reject = 1;
+		if (*priv->priv_rcpt == 0)
+			strcpy(priv->priv_rcpt, rcpt);
+		else
+			strcpy(priv->priv_rcpt, "(multiple recipients)");
+		return SMFIS_CONTINUE;
 	}
 
+	/*
+	 * Log temporary failure and report to the client.
+	 */
+	log_and_report_greylisting(ctx, priv, *envrcpt);
 	return SMFIS_TEMPFAIL;
 }
 
@@ -421,13 +424,18 @@ mlfi_eom(ctx)
 	char timestr[HDRLEN + 1];
 	char tzstr[HDRLEN + 1];
 	char tznamestr[HDRLEN + 1];
-	char *whystr = NULL;
+	char whystr [HDRLEN + 1];
 	char host[ADDRLEN + 1];
 	time_t t;
 	struct tm ltm;
 
 	priv = (struct mlfi_priv *) smfi_getpriv(ctx);
 
+	if (priv->priv_delayed_reject) {
+		log_and_report_greylisting(ctx, priv, priv->priv_rcpt);
+		return SMFIS_TEMPFAIL;
+	}
+
 	if ((fqdn = smfi_getsymval(ctx, "{j}")) == NULL) {
 		syslog(LOG_DEBUG, "smfi_getsymval failed for {j}");
 		gethostname(host, ADDRLEN);
@@ -464,48 +472,59 @@ mlfi_eom(ctx)
 		if ((conf.c_report & C_NODELAYS) == 0)
 			return SMFIS_CONTINUE;
 			
-		switch (priv->priv_whitelist) {
-		case EXF_DOMAIN:
-			whystr = "Sender DNS name whitelisted";
-			break;
-
-		case EXF_ADDR:
-			whystr = "Sender IP whitelisted";
-			break;
-
-		case EXF_FROM:
-			whystr = "Sender e-mail whitelisted";
-			break;
-
-		case EXF_AUTH:
-			whystr = "Sender succeded SMTP AUTH authentication";
-			break;
-
-		case EXF_SPF:
-			whystr = "Sender is SPF-compliant";
-			break;
-
-		case EXF_NONIPV4:
-			whystr = "Message not sent from an IPv4 address";
-			break;
-
-		case EXF_STARTTLS:
-			whystr = "Sender succeeded STARTTLS authentication";
-			break;
-
-		case EXF_RCPT:
-			whystr = "Recipient e-mail whitelisted";
-			break;
-
-		case EXF_AUTO:
-			whystr = "IP, sender and recipient auto-whitelisted";
-			break;
-
-		default:
-			syslog(LOG_ERR, "%s: unexpected priv_whitelist = %d", 	
+		whystr[0] = '\0';
+		if (priv->priv_whitelist & EXF_DOMAIN) {
+			ADD_REASON(whystr, "Sender DNS name whitelisted");
+			priv->priv_whitelist &= ~EXF_DOMAIN;
+		}
+		if (priv->priv_whitelist & EXF_ADDR) {
+			ADD_REASON(whystr, "Sender IP whitelisted");
+			priv->priv_whitelist &= ~EXF_ADDR;
+		}
+		if (priv->priv_whitelist & EXF_FROM) {
+			ADD_REASON(whystr, "Sender e-mail whitelisted");
+			priv->priv_whitelist &= ~EXF_FROM;
+		}
+		if (priv->priv_whitelist & EXF_AUTH) {
+			ADD_REASON(whystr, 
+			    "Sender succeeded SMTP AUTH authentication");
+			priv->priv_whitelist &= ~EXF_AUTH;
+		}
+		if (priv->priv_whitelist & EXF_ACCESSDB) {
+			ADD_REASON(whystr, 
+			    "Message whitelisted by Sendmail access database");
+			priv->priv_whitelist &= ~EXF_ACCESSDB;
+		}
+		if (priv->priv_whitelist & EXF_SPF) {
+			ADD_REASON(whystr, "Sender is SPF-compliant");
+			priv->priv_whitelist &= ~EXF_SPF;
+		}
+		if (priv->priv_whitelist & EXF_NONIPV4) {
+			ADD_REASON(whystr, 
+			    "Message not sent from an IPv4 address");
+			priv->priv_whitelist &= ~EXF_NONIPV4;
+		}
+		if (priv->priv_whitelist & EXF_STARTTLS) {
+			ADD_REASON(whystr, "Sender succeeded STARTTLS authentication");
+			priv->priv_whitelist &= ~EXF_STARTTLS;
+		}
+		if (priv->priv_whitelist & EXF_RCPT) {
+			ADD_REASON(whystr, "Recipient e-mail whitelisted");
+			priv->priv_whitelist &= ~EXF_RCPT;
+		}
+		if (priv->priv_whitelist & EXF_AUTO) {
+			ADD_REASON(whystr, "IP, sender and recipient auto-whitelisted");
+			priv->priv_whitelist &= ~EXF_AUTO;
+		}
+		if (priv->priv_whitelist & EXF_DEFAULT) {
+			ADD_REASON(whystr, "Default is to whitelist mail");
+			priv->priv_whitelist &= ~EXF_DEFAULT;
+		}
+		priv->priv_whitelist &= ~(EXF_GREYLIST | EXF_WHITELIST);
+		if (priv->priv_whitelist != 0) {
+			syslog(LOG_ERR, "%s: unexpected priv_whitelist = %d",
 			    priv->priv_queueid, priv->priv_whitelist);
-			whystr = "Internal error";
-			break;
+			strncat (whystr, "Internal error ", HDRLEN);
 		}
 
 		snprintf(hdr, HDRLEN, "%s, not delayed by "
@@ -563,7 +582,7 @@ main(argc, argv)
 	char *argv[];
 {
 	int ch;
-
+	int checkonly = 0;
 	/*
 	 * Load configuration defaults
 	 */
@@ -572,7 +591,7 @@ main(argc, argv)
 	/* 
 	 * Process command line options 
 	 */
-	while ((ch = getopt(argc, argv, "Aa:vDd:qw:f:hp:P:Tu:rSL:M:")) != -1) {
+	while ((ch = getopt(argc, argv, "Aa:cvDd:qw:f:hp:P:Tu:rSL:M:l")) != -1) {
 		switch (ch) {
 		case 'A':
 			defconf.c_noauth = 1;
@@ -589,6 +608,9 @@ main(argc, argv)
 			    (time_t)humanized_atoi(optarg);
 			defconf.c_forced |= C_AUTOWHITE;
 			break;
+		case 'c':
+		        checkonly = 1;
+			break;
 
 		case 'D':
 			defconf.c_nodetach = 1;
@@ -748,6 +770,11 @@ main(argc, argv)
 			defconf.c_forced |= C_TESTMODE;
 			break;
 
+		case 'l':
+			defconf.c_acldebug = 1;
+			defconf.c_forced |= C_ACLDEBUG;
+			break;
+
 		case 'h':
 		default:
 			usage(argv[0]);
@@ -755,6 +782,10 @@ main(argc, argv)
 		}
 	}
 	
+	if (checkonly) {
+		conf_load();
+		exit(0);
+	}
 	/* 
 	 * Register our callbacks 
 	 */
@@ -767,7 +798,7 @@ main(argc, argv)
 	 * Various init
 	 */
 	conf_init();
-	except_init();
+	acl_init ();
 	pending_init();
 	peer_init();
 	autowhite_init();
@@ -788,7 +819,7 @@ main(argc, argv)
 
 	
 	if (conf.c_socket == NULL) {
-		fprintf(stderr, "%s: No socket provided, exitting\n", argv[0]);
+		fprintf(stderr, "%s: No socket provided, exiting\n", argv[0]);
 		usage(argv[0]);
 	}
 	cleanup_sock(conf.c_socket);
@@ -808,9 +839,9 @@ main(argc, argv)
 		(void)close(0);
 		(void)open("/dev/null", O_RDONLY, 0);
 		(void)close(1);
-		(void)open("/dev/null", O_RDONLY, 0);
+		(void)open("/dev/null", O_WRONLY, 0);
 		(void)close(2);
-		(void)open("/dev/null", O_RDONLY, 0);
+		(void)open("/dev/null", O_WRONLY, 0);
 
 		if (chdir("/") != 0) {
 			fprintf(stderr, "%s: cannot chdir to root: %s\n",
@@ -853,11 +884,28 @@ main(argc, argv)
 		struct passwd *pw = NULL;
 
 		if ((pw = getpwnam(conf.c_user)) == NULL) {
-			syslog(LOG_ERR, "%s: Cannot get user %s data: %s\n",
+			syslog(LOG_ERR, "%s: cannot get user %s data: %s\n",
 			    argv[0], conf.c_user, strerror(errno));
 			exit(EX_OSERR);
 		}
 
+#ifdef HAVE_INITGROUPS
+		if (initgroups(conf.c_user, pw->pw_gid) != 0) {
+		        syslog(LOG_ERR, "%s: cannot change "
+			    "supplementary groups: %s\n",
+			    argv[0], strerror(errno));
+			exit(EX_OSERR);
+		}
+#endif
+
+		if (setgid(pw->pw_gid) != 0 ||
+		    setegid(pw->pw_gid) != 0) {
+			syslog(LOG_ERR, "%s: cannot change GID: %s\n",
+			    argv[0], strerror(errno));
+			exit(EX_OSERR);
+		}
+
+
 		if ((setuid(pw->pw_uid) != 0) ||
 		    (seteuid(pw->pw_uid) != 0)) {
 			syslog(LOG_ERR, "%s: cannot change UID: %s\n",
@@ -886,6 +934,12 @@ main(argc, argv)
 		exit(EX_OSERR);
 	}	
 
+	/*
+	 * Dump the ACL for debugging purposes
+	 */
+	if (conf.c_debug || conf.c_acldebug)
+		acl_dump();
+
 	/*
 	 * Here we go!
 	 */
@@ -897,8 +951,10 @@ usage(progname)
 	char *progname;
 {
 	fprintf(stderr, 
-	    "usage: %s [-ADvqST] [-a autowhite] [-d dumpfile] [-f configfile]\n"
-	    "       [-w delay] [-u username] [-L cidrmask] -p socket\n", 
+	    "usage: %s [-A] [-a autowhite_delay] [-c] [-D] [-d dumpfile]\n"
+	    "       [-f configfile] [-h] [-l] [-q] [-r] [-S] [-T]\n"
+	    "       [-u username] [-v] [-w greylist_delay] [-L cidrmask]\n"
+	    "       [-M prefixlen] [-P pidfile] -p socket\n", 
 	    progname);
 	exit(EX_USAGE);
 }
@@ -1053,7 +1109,7 @@ prefix2mask4(cidr, mask)
 		bzero((void *)mask, sizeof(*mask));
 	} else {
 		cidr = 32 - cidr;
-		*mask = inet_makeaddr(~((1UL << cidr) - 1), 0L);
+		mask->s_addr = htonl(~((1UL << cidr) - 1));
 	}
 	
 	return mask;
@@ -1094,8 +1150,8 @@ unmappedaddr(sa, salen)
 {
 #ifdef AF_INET6
 	struct in_addr addr4;
-	int port;
-
+	int port;       
+			
 	if (SA6(sa)->sin6_family != AF_INET6 ||
 	    !IN6_IS_ADDR_V4MAPPED(SADDR6(sa)))
 		return;
@@ -1113,6 +1169,54 @@ unmappedaddr(sa, salen)
 	return;
 }
 
+void
+log_and_report_greylisting(ctx, priv, rcpt)
+	SMFICTX *ctx;
+	struct mlfi_priv *priv;
+	char *rcpt;
+{
+	int h, mn, s;
+	char hdr[HDRLEN + 1];
+	char addrstr[IPADDRSTRLEN];
+	time_t remaining;
+	char *delayed_rj;
+
+	/*
+	 * The message has been added to the greylist and will be delayed.
+	 * Log this and report to the client.
+	 */
+	iptostring(SA(&priv->priv_addr), priv->priv_addrlen, addrstr,
+	    sizeof(addrstr));
+
+	remaining = priv->priv_remaining;
+	h = remaining / 3600;
+	remaining = remaining % 3600;
+	mn = (remaining / 60);
+	remaining = remaining % 60;
+	s = remaining;
+
+	if (priv->priv_delayed_reject)
+		delayed_rj = " after DATA phase";
+	else
+		delayed_rj = "";
+
+	syslog(LOG_INFO, "%s: addr %s from %s to %s delayed%s for %02d:%02d:%02d",
+	    priv->priv_queueid, addrstr, priv->priv_from, rcpt, delayed_rj,
+	    h, mn, s);
+
+	if (conf.c_quiet) {
+		(void)smfi_setreply(ctx, "451", "4.7.1",
+		    "Greylisting in action, please come back later");
+	} else {
+		snprintf(hdr, HDRLEN,
+		    "Greylisting in action, please come "
+		    "back in %02d:%02d:%02d", h, mn, s);
+		(void)smfi_setreply(ctx, "451", "4.7.1", hdr);
+	}
+
+	return;
+}
+
 void
 final_dump(void) {
 
@@ -1123,6 +1227,6 @@ final_dump(void) {
 		syslog(LOG_INFO, "Final database dump: no change to dump");
 	}
 
-	syslog(LOG_INFO, "Exitting");
+	syslog(LOG_INFO, "Exiting");
 	return;
 }
