@@ -1,4 +1,4 @@
-/* $Id: sync.c,v 1.49 2004/08/09 20:26:52 manu Exp $ */
+/* $Id: sync.c,v 1.56 2005/10/20 07:24:53 manu Exp $ */
 
 /*
  * Copyright (c) 2004 Emmanuel Dreyfus
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID
-__RCSID("$Id: sync.c,v 1.49 2004/08/09 20:26:52 manu Exp $");
+__RCSID("$Id: sync.c,v 1.56 2005/10/20 07:24:53 manu Exp $");
 #endif
 #endif
 
@@ -72,6 +72,9 @@ pthread_rwlock_t sync_lock; /* For all peer's sync queue */
 pthread_cond_t sync_sleepflag;
 
 static void sync_listen(char *, char *, struct sync_master_sock *);
+static int local_addr(const struct sockaddr *sa, const socklen_t salen);
+static int sync_queue_poke(struct peer *, struct sync *);
+static struct sync * sync_queue_peek(struct peer *);
 
 void
 peer_init(void) {
@@ -105,16 +108,12 @@ peer_clear(void) {
 	struct sync *sync;
 
 	PEER_WRLOCK;
-	SYNC_WRLOCK;
 
 	while(!LIST_EMPTY(&peer_head)) {
 		peer = LIST_FIRST(&peer_head);
-		while(!TAILQ_EMPTY(&peer->p_deferred)) {
-			sync = TAILQ_FIRST(&peer->p_deferred);
-			TAILQ_REMOVE(&peer->p_deferred, sync, s_list);
-			peer->p_qlen--;
+
+		while((sync = sync_queue_peek(peer)) != NULL)
 			sync_free(sync);
-		}
 			
 		if (peer->p_stream != NULL)
 			fclose(peer->p_stream);
@@ -124,7 +123,6 @@ peer_clear(void) {
 		free(peer);
 	}
 
-	SYNC_UNLOCK;
 	PEER_UNLOCK;
 
 	return;	
@@ -144,6 +142,7 @@ peer_add(peername)
 
 	peer->p_qlen = 0;
 	peer->p_stream = NULL;
+	peer->p_flags = 0;
 	TAILQ_INIT(&peer->p_deferred);
 
 	PEER_WRLOCK;
@@ -194,6 +193,39 @@ out:
 	return;
 }
 
+static int
+sync_queue_poke(peer, sync)
+	struct peer *peer;
+	struct sync *sync;
+{
+	SYNC_WRLOCK;
+	if (peer->p_qlen < SYNC_MAXQLEN) {
+		TAILQ_INSERT_HEAD(&peer->p_deferred, sync, s_list);
+		peer->p_qlen++;
+		SYNC_UNLOCK;
+		return 1;
+	} else {
+		SYNC_UNLOCK;
+		return 0;
+	}
+}
+
+static struct sync *
+sync_queue_peek(peer)
+	struct peer *peer;
+{
+	struct sync *sync;
+
+	SYNC_WRLOCK;
+	sync = TAILQ_FIRST(&peer->p_deferred);
+	if (!TAILQ_EMPTY(&peer->p_deferred)) {
+		TAILQ_REMOVE(&peer->p_deferred, sync, s_list);
+		peer->p_qlen--;
+	}
+	SYNC_UNLOCK;
+	return sync;
+}
+
 int
 sync_send(peer, type, pending)	/* peer list is read-locked */
 	struct peer *peer;
@@ -313,6 +345,21 @@ peer_connect(peer)	/* peer list is read-locked */
 		    peer->p_name, gai_strerror(err), peer->p_qlen);
 		return -1;
 	}
+
+	for (res = res0; res; res = res->ai_next) {
+		/*We only test an address family which kernel supports. */
+		s = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
+		if (s == -1)
+			continue;
+		close(s);
+
+		if (local_addr(res->ai_addr, res->ai_addrlen)) {
+			peer->p_flags |= P_LOCAL;
+			freeaddrinfo(res0);
+			return -1;
+		}
+	}
+
 	for (res = res0; res; res = res->ai_next) {
 		s = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
 		if (s == -1)
@@ -358,6 +405,12 @@ peer_connect(peer)	/* peer list is read-locked */
 		syslog(LOG_ERR, "cannot sync, invalid address");
 		return -1;
 	}
+
+	if (local_addr(SA(&raddr), raddrlen)) {
+		peer->p_flags |= P_LOCAL;
+		return -1;
+	}
+
 	switch (SA(&raddr)->sa_family) {
 	case AF_INET:
 		SA4(&raddr)->sin_port = service;
@@ -505,19 +558,35 @@ sync_master_restart(void) {
 		    strerror(errno));
 		exit(EX_OSERR);
 	}
-	if (sync_master6.runs &&
-	    (error = pthread_create(&tid, NULL, sync_master,
-	    (void *)&sync_master6)) != 0) {
-		syslog(LOG_ERR, "Cannot run MX sync thread for IPv6: %s\n",
-		    strerror(error));
-		exit(EX_OSERR);
+	if (sync_master6.runs) {
+		if ((error = pthread_create(&tid, NULL, sync_master,
+		    (void *)&sync_master6)) != 0) {
+			syslog(LOG_ERR, 
+			    "Cannot run MX sync thread for IPv6: %s\n",
+			    strerror(error));
+			exit(EX_OSERR);
+		}
+		if ((error = pthread_detach(tid)) != 0) {
+			syslog(LOG_ERR, 
+			    "pthread_detach failed for IPv6 MX sync: %s\n",
+			    strerror(error));
+			exit(EX_OSERR);
+		}
 	}
-	if (sync_master4.runs &&
-	    (error = pthread_create(&tid, NULL, sync_master,
-	    (void *)&sync_master4)) != 0) {
-		syslog(LOG_ERR, "Cannot run MX sync thread for IPv4: %s\n",
-		    strerror(error));
-		exit(EX_OSERR);
+	if (sync_master4.runs) {
+		if ((error = pthread_create(&tid, NULL, sync_master,
+		    (void *)&sync_master4)) != 0) {
+			syslog(LOG_ERR, 
+			    "Cannot run MX sync thread for IPv4: %s\n",
+			    strerror(error));
+			exit(EX_OSERR);
+		}
+		if ((error = pthread_detach(tid)) != 0) {
+			syslog(LOG_ERR, 
+			    "pthread_detach failed for IPv4 MX sync: %s\n",
+			    strerror(error));
+			exit(EX_OSERR);
+		}
 	}
 }
 
@@ -641,6 +710,12 @@ sync_master(arg)
 			fclose(stream);
 			continue;
 		}
+		if ((error = pthread_detach(tid)) != 0) {
+			syslog(LOG_ERR, "incoming connexion from %s failed, "
+			    "pthread_detach failed: %s",
+			    peerstr, strerror(error));
+			exit(EX_OSERR);
+		}
 	}
 
 	/* NOTREACHED */
@@ -991,30 +1066,27 @@ sync_queue(peer, type, pending)	/* peer list must be read-locked */
 	int error;
 	struct sync *sync;
 
+	if (peer->p_flags & P_LOCAL)
+		return;
+
+	if ((sync = malloc(sizeof(*sync))) == NULL) {
+		syslog(LOG_ERR, "cannot allocate memory: %s", 
+		    strerror(errno)); 
+		exit(EX_OSERR);
+	}
+
+	sync->s_peer = peer;
+	sync->s_type = type;
+	sync->s_pending = pending_ref(pending);
+
 	/*
 	 * If the queue has overflown, try to wakeup sync_sender to
 	 * void it, but do not accept new entries anymore.
 	 */
-	if (peer->p_qlen >= SYNC_MAXQLEN) {
+	if (!sync_queue_poke(peer, sync)) {
 		syslog(LOG_ERR, "peer %s queue overflow (%d entries), "
 		    "discarding new entry", peer->p_name, peer->p_qlen);
-
-	} else {
-
-		if ((sync = malloc(sizeof(*sync))) == NULL) {
-			syslog(LOG_ERR, "cannot allocate memory: %s", 
-			    strerror(errno)); 
-			exit(EX_OSERR);
-		}
-
-		sync->s_peer = peer;
-		sync->s_type = type;
-		sync->s_pending = pending_ref(pending);
-
-		SYNC_WRLOCK;
-		TAILQ_INSERT_HEAD(&peer->p_deferred, sync, s_list);
-		SYNC_UNLOCK;
-		peer->p_qlen++;
+		sync_free(sync);
 	}
 
 	if ((error = pthread_cond_signal(&sync_sleepflag)) != 0) {
@@ -1043,6 +1115,10 @@ sync_sender_start(void) {
 		syslog(LOG_ERR, "pthread_create failed: %s", strerror(error));
 		exit(EX_OSERR);
 	}
+	if ((error = pthread_detach(tid)) != 0) {
+		syslog(LOG_ERR, "pthread_detach failed: %s", strerror(error));
+		exit(EX_OSERR);
+	}
 	return;
 }
 
@@ -1085,30 +1161,20 @@ sync_sender(dontcare)
 			goto out;
 			
 		LIST_FOREACH(peer, &peer_head, p_list) {
-			/* XXX take a read lock and then upgrade it */
-			while (TAILQ_EMPTY(&peer->p_deferred) == 0) {
-				SYNC_WRLOCK;
-				sync = TAILQ_FIRST(&peer->p_deferred);
-				TAILQ_REMOVE(&peer->p_deferred, sync, s_list);
-				SYNC_UNLOCK;
+			/* Don't try to sync with ourselves */
+			if (peer->p_flags & P_LOCAL)
+				continue;
+
+			while ((sync = sync_queue_peek(peer)) != NULL ) {
 
-				/* 
-				 * We cannot keep the lock in sync_send
-				 * as the send might timeout. Hence we
-				 * remove the item, and put it back if 
-				 * the send operation fails.
-				 */
 				if (sync_send(sync->s_peer, sync->s_type, 
 				    sync->s_pending) != 0) {
-					SYNC_WRLOCK;
-					TAILQ_INSERT_HEAD(&peer->p_deferred, 
-					    sync, s_list);
-					SYNC_UNLOCK;
+					if (!sync_queue_poke(peer, sync))
+						sync_free(sync);
 
 					break;
 				}
 
-				peer->p_qlen--;
 				sync_free(sync);
 
 				done++;
@@ -1126,3 +1192,58 @@ out:
 		}
 	}
 }
+
+
+static int
+local_addr(sa, salen)
+	const struct sockaddr *sa;
+	const socklen_t salen;
+{
+	sockaddr_t addr;
+	int	sfd, islocal;
+
+	memcpy(&addr, sa, salen);
+	switch(sa->sa_family) {
+	case AF_INET:
+		SA4(&addr)->sin_port = 0;
+		break;
+
+#ifdef AF_INET6
+	case AF_INET6:
+		SA6(&addr)->sin6_port = 0;
+		break;
+#endif
+
+	default:
+		syslog(LOG_ERR, "local_addr: unsupported AF %d\n",
+		    sa->sa_family);
+		return -1;
+		break;
+	}
+
+	if ((sfd = socket(sa->sa_family, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
+		syslog(LOG_ERR, "local_addr: socket failed: %s\n",
+		    strerror(errno));
+		return -1;
+	}
+
+	if (bind(sfd, sa, salen) == -1) {
+		if (errno != EADDRNOTAVAIL && 
+#ifdef __FreeBSD__
+		    errno != EINVAL &&
+#endif
+		    1) {
+			syslog(LOG_ERR, "local_addr: bind failed: %s\n",
+			    strerror(errno));
+			islocal = -1;
+		} else {
+			islocal = 0;
+		}
+	} else {
+		islocal = 1;
+	}
+
+	close(sfd);
+
+	return islocal;
+}
